/*
  Originally named "Arduino code for dynamic playseat 2DOF"
  Created 24 May 2011 by Jim Lindblom   SparkFun Electronics https://www.sparkfun.com/products/10182 "Example Code"
  Created 24 Apr 2012 by Jean David SEDRUE Version betatest26 - 24042012 http://www.gamoover.net/Forums/index.php?topic=25907
  Updated 20 May 2013 by Mathieu RIVIER in english http://www.xsimulator.net/2dof-wiper-motor-playseat/ in french : http://www.gamoover.net/Forums/index.php?topic=27617

  Forked 09 Nov 2013 by Daan Kortenbach from http://www.xsimulator.net/community/marketplace/arduino-code-2dof-and-2-pins-direction-hbridge.67/
  For furthers changes see README.md and https://github.com/daankortenbach/arduino-2dof-for-simtools
 */


/*
 * DEFINITIONS
 */
#define BRAKEVCC 0

// Beware it's depending on your hardware wiring
#define RV 2

// Beware it's depending on your hardware wiring
#define FW 1

#define STOP 0
#define BRAKEGND 3

// 255 or less, if you want to lower the maximum motor's speed
#define pwmMax 255

// Defining the range of potentiometer's rotation
const int potMini = 208;
const int potMaxi = 815;

#define motLeft 0
#define motRight 1
#define potL A0
#define potR A1


/*
 * VNH2SP30 pin definitions
 */

// INA: Clockwise input
int inApin[2] = {
  7, 4
};

// INB: Counter-clockwise input
int inBpin[2] = {
  8, 9
};

// PWM input
int pwmpin[2] = {
  5, 6
};

// CS: Current sense ANALOG input
int cspin[2] = {
  2, 3
};

// EN: Status of switches output (Analog pin)
int enpin[2] = {
  0, 1
};

// Not explained by Sparkfun
int statpin = 13;


/*
 * Init position value
 */

// Middle position 0-1024
int DataValueL = 512;

// Middle position 0-1024
int DataValueR = 512;


/**
 * INITIALIZATION
 */
void setup()
{
  // Serial initialization
  Serial.begin( 115200 );

  // Initialization of Arduino's pins

  // Not explained by Sparkfun
  pinMode( statpin, OUTPUT );

  digitalWrite( statpin, LOW );

  for ( int i = 0; i < 2; i++ )
  {
    pinMode( inApin[i], OUTPUT );
    pinMode( inBpin[i], OUTPUT );
    pinMode( pwmpin[i], OUTPUT );
  }

  // Initialize braked for motor
  for ( int i = 0; i < 2; i++ )
  {
    digitalWrite( inApin[i], LOW );
    digitalWrite( inBpin[i], LOW );
  }
}


/**
 * Main Loop
 */
void loop()
{
  int sensorL, sensorR;

  // DataValueR & L contain the last order received (if there is no newer received, the last is kept)
  readSerialData();
  // The previous order will still be used by the PID regulation MotorMotion Function

  // Range 0-1024
  sensorR = analogRead( potR );

  // Range 0-1024
  sensorL = analogRead( potL );

  motorMotion( motRight, sensorR, DataValueR );
  motorMotion( motLeft, sensorL, DataValueL );
}


/**
 * Procedure: wait for complete trame
 */
void readSerialData()
{
  // keep this function short, because the loop has to be short to keep the control over the motors
  byte Data[3] = {
    '0','0','0'
  };

  if ( Serial.available() > 2 ) {
    // Parse the buffer : test if the byte is the first of the order "R"
    Data[0] = Serial.read();

    if ( Data[0] == 'R' ) {
      Data[1] = Serial.read();
      Data[2] = Serial.read();

      //  Call the function that converts the hexa in decimal and that maps the range
      DataValueR = NormalizeData( Data );
    }

    if ( Data[0] == 'L' ) {
      Data[1] = Serial.read();
      Data[2] = Serial.read();

      // Call the function that converts the hexa in decimal and maps the range
      DataValueL = NormalizeData( Data );
    }
  }
  if ( Serial.available() > 16 ) {
    Serial.flush();
  }
}


/**
 * Motor motion
 * @param numMot    [description]
 * @param actualPos [description]
 * @param targetPos [description]
 */
void motorMotion( int numMot, int actualPos, int targetPos )
{
  // No order to move will be sent to the motor if the target is close to the actual position
  // this prevents short jittering moves
  // could be a parameter read from a pot on an analogic pin
  // the highest value, the calmest the simulator would be (less moves)
  int Tol = 20;

  int gap;
  int pwm;
  int brakingDistance = 30;

  // Security concern : targetPos has to be within the mechanically authorized range
  targetPos = constrain( targetPos, potMini + brakingDistance, potMaxi - brakingDistance );

  gap = abs( targetPos - actualPos );

  if ( gap <= Tol) {
    // Too near to move
    motorOff( numMot );
  }
  else {
    // PID : calculates speed according to distance
    pwm = 195;

    if ( gap > 50 ) pwm = 215;
    if ( gap > 75 ) pwm = 235;
    if ( gap > 100) pwm = 255;

    // Adjust the value according to pwmMax for mechanical debugging purpose !
    pwm = map( pwm, 0, 255, 0, pwmMax );

    // If motor is outside from the range, send motor back to the limit !

    // Go forward (up)
    if ( ( actualPos < potMini ) || ( actualPos < targetPos ) ) {
      motorGo( numMot, FW, pwm );
    }

    // Go reverse (down)
    if ( ( actualPos > potMaxi ) || ( actualPos > targetPos ) ) {
      motorGo( numMot, RV, pwm );
    }
  }
}


/**
 * Brake Ground : free wheel actually
 * @param motor [description]
 */
void motorOff( int motor )
{
  digitalWrite( inApin[motor], LOW );
  digitalWrite( inBpin[motor], LOW );
  analogWrite( pwmpin[motor], 0 );
}


/**
 * "brake VCC" : short-circuit inducing electromagnetic brake
 * @param motor [description]
 */
void motorOffBraked( int motor )
{
  digitalWrite( inApin[motor], HIGH );
  digitalWrite( inBpin[motor], HIGH );
  analogWrite( pwmpin[motor], 0 );
}


/**
 * [motorGo description]
 * @param motor  [description]
 * @param direct [description]
 * @param pwm    [description]
 */
void motorGo( uint8_t motor, uint8_t direct, uint8_t pwm )
{
  if ( motor <= 1 ) {
    if ( direct <= 4) {

      // Set inA[motor]
      if ( direct <= 1)
        digitalWrite( inApin[motor], HIGH );
      else
        digitalWrite( inApin[motor], LOW );

      // Set inB[motor]
      if ( ( direct == 0) || ( direct == 2 ) )
        digitalWrite( inBpin[motor], HIGH );
      else
        digitalWrite( inBpin[motor], LOW );

      analogWrite( pwmpin[motor], pwm );

    }
  }
}


/**
 * [motorDrive description]
 *
 * more readable function than Jim's (for educational purpose)
 * but 50 octets heavier ->  unused
 *
 * @param motor  [description]
 * @param direct [description]
 * @param pwm    [description]
 */
void motorDrive( uint8_t motor, uint8_t direct, uint8_t pwm )
{
  if ( motor <= 1 && direct <= 4 ) {
    switch ( direct ) {

      // Electromagnetic brake : brake VCC
      case 0:
        digitalWrite( inApin[motor], HIGH );
        digitalWrite( inBpin[motor], HIGH );
        break;

      // Brake Ground (free wheel)
      case 3:
        digitalWrite(inApin[motor], LOW);
        digitalWrite(inBpin[motor], LOW);
        break;

      // Forward : beware it's depending on your hardware wiring
      case 1:
        digitalWrite( inApin[motor], HIGH );
        digitalWrite( inBpin[motor], LOW );
        break;

      // Reverse : beware it's depending on your hardware wiring
      case 2:
        digitalWrite( inApin[motor], LOW );
        digitalWrite( inBpin[motor], HIGH );
        break;
    }
    analogWrite( pwmpin[motor], pwm );
  }
}


/**
 * [testPot description]
 */
void testPot()
{
  Serial.print( analogRead( A4 ) );
  Serial.print( ";" );
  Serial.println( analogRead( A5 ) );
  delay( 250 );

}


/**
 * [testpulse description]
 */
void testpulse()
{
  int pw = 120;
  while ( true ) {

    motorGo( motLeft, FW, pw );
    delay( 250 );
    motorOff( motLeft );
    delay( 250 );
    motorGo( motLeft, RV, pw );
    delay( 250 );
    motorOff( motLeft );

    delay( 500 );

    motorGo( motRight, FW, pw );
    delay( 250 );
    motorOff( motRight );
    delay( 250 );
    motorGo( motRight, RV, pw );
    delay( 250 );
    motorOff( motRight );
    Serial.println( "testpulse pwm:80" );
    delay( 500 );

  }
}


/*
 * convert Hex to Dec
 */
int NormalizeData( byte x[3] )
{
  int result;

  // Only a LSB and Carrier Return
  if ( x[2] == 13 ) {

    // Move MSB to LSB
    x[2] = x[1];

    // Clear MSB
    x[1] = '0';
  }

  for ( int i = 1; i < 3; i++ ) {

    // For xA to xF
    if ( x[i] > 47 && x[i] < 58 ) {
      x[i] = x[i] - 48;
    }

    // For x0 to x9
    if ( x[i] > 64 && x[i] < 71 ) {
      x[i] = x[i] - 55;
    }
  }

  // Map the range from Xsim (0 <-> 255) to the mechanically authorized range (potMini <-> potMaxi)
  result = map( ( x[1] * 16 + x[2] ), 0, 255, potMini, potMaxi );

  return result;
}
